import sys

import click


def raise_error(message, exit_code=1):
    click.echo('Error: %s' % message, file=sys.stderr)
    sys.exit(exit_code)


def assert_type(value, expected_type):
    if type(value) != expected_type:
        raise_error(
            'Expected a %s, got %s instead' % (
                expected_type.__name__, type(value).__name__))



import click
from beautifultable import BeautifulTable

from pytest_django_queries.utils import assert_type, raise_error


def format_underscore_name_to_human(name):
    if name.startswith('test'):
        _, name = name.split('test', 1)
    return name.replace('_', ' ')


class TestEntryData(object):
    BASE_FIELDS = [
        ('test_name', 'Test Name')
    ]
    REQUIRED_FIELDS = [
        ('query-count', 'Queries'),
    ]
    FIELDS = BASE_FIELDS + REQUIRED_FIELDS

    def __init__(self, test_name, data):
        """
        :param data: The test entry's data.
        :type data: dict
        """

        assert_type(data, dict)

        self._raw_data = data
        self.test_name = test_name

        for field, _ in self.REQUIRED_FIELDS:
            setattr(self, field, self._get_required_key(field))

    def _get_required_key(self, key):
        if key in self._raw_data:
            return self._raw_data.get(key)
        raise_error('Got invalid data. It is missing a required key: %s' % key)


def iter_entries(entries):
    for module_name, module_data in sorted(entries.items()):
        assert_type(module_data, dict)

        yield module_name, (
            TestEntryData(test_name, test_data)
            for test_name, test_data in sorted(module_data.items()))


def print_entries(data):
    table = BeautifulTable()
    table.column_headers = ['Module', 'Tests']
    for module_name, module_entries in iter_entries(data):
        subtable = BeautifulTable()
        subtable.column_headers = [field for _, field in TestEntryData.FIELDS]
        for entry in module_entries:
            subtable.append_row([
                getattr(entry, field) for field, _ in TestEntryData.FIELDS])
        table.append_row([module_name, subtable])
    click.echo(table)


def print_entries_as_html(data, template):
    html_content = template.render(
        data=iter_entries(data), humanize=format_underscore_name_to_human)
    click.echo(html_content, nl=False)



import json
from os.path import abspath, dirname
from os.path import join as pathjoin

import click
from jinja2 import Template
from jinja2 import exceptions as jinja_exceptions

from pytest_django_queries.plugin import DEFAULT_RESULT_FILENAME
from pytest_django_queries.tables import print_entries, print_entries_as_html

HERE = dirname(__file__)
DEFAULT_TEMPLATE_PATH = abspath(pathjoin(HERE, 'templates', 'default_bootstrap.jinja2'))


class JsonFileParamType(click.File):
    name = 'integer'

    def convert(self, value, param, ctx):
        fp = super(JsonFileParamType, self).convert(value, param, ctx)
        try:
            loaded = json.load(fp)
            if type(loaded) is not dict:
                self.fail('The file is not a dictionary', param, ctx)
            return loaded
        except ValueError as e:
            self.fail(
                'The file is not valid json: %s' % str(e),
                param,
                ctx)


class Jinja2TemplateFile(click.File):
    name = 'integer'

    def convert(self, value, param, ctx):
        fp = super(Jinja2TemplateFile, self).convert(value, param, ctx)
        try:
            return Template(fp.read())
        except jinja_exceptions.TemplateError as e:
            self.fail(
                'The file is not a valid jinja2 template: %s' % str(e),
                param,
                ctx)


@click.group()
def main():
    """Command line tool for pytest-django-queries."""


@main.command()
@click.argument(
    'input_file', type=JsonFileParamType('r'), default=DEFAULT_RESULT_FILENAME)
def show(input_file):
    """View a given rapport."""
    return print_entries(input_file)


@main.command()
@click.argument(
    'input_file', type=JsonFileParamType('r'), default=DEFAULT_RESULT_FILENAME)
@click.option('--template', type=Jinja2TemplateFile('r'), default=DEFAULT_TEMPLATE_PATH)
def html(input_file, template):
    """Render the results as HTML instead of a raw table."""
    return print_entries_as_html(input_file, template)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from os.path import isfile
from setuptools import setup


def _read_file(path):
    with open(path) as fp:
        return fp.read().strip()


PROJECT_VERSION = _read_file('VERSION.txt')

REQUIREMENTS = _read_file('requirements.txt').split('\n')
DEV_REQUIREMENTS = _read_file('requirements_dev.txt').split('\n')


if isfile('README.md'):
   long_description = _read_file('README.md')
else:
    long_description = ''

setup(
    name='pytest-django-queries',
    author='NyanKiyoshi',
    author_email='hello@vanille.bid',
    url='https://github.com/NyanKiyoshi/pytest-django-queries/',
    description='Generate performance rapports from your django database '
                'performance tests.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version=PROJECT_VERSION,
    packages=['pytest_django_queries'],
    include_package_data=True,
    entry_points={
        'pytest11': ['django_queries = pytest_django_queries.plugin'],
        'console_scripts': ['django-queries = pytest_django_queries.cli:main']},
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Framework :: Django',
        'Framework :: Pytest',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    install_requires=REQUIREMENTS,
    extras_require={'dev': DEV_REQUIREMENTS},
    zip_safe=False)



import json

import pytest
from django.test.utils import CaptureQueriesContext

# Defines the plugin marker name
PYTEST_QUERY_COUNT_MARKER = 'count_queries'
DEFAULT_RESULT_FILENAME = '.pytest-queries'


def _set_session(config, new_session):
    config.pytest_django_queries_session = new_session


def _get_session(request):
    return request.config.pytest_django_queries_session


class _Session(object):
    def __init__(self, save_path):
        self.save_path = save_path
        self._data = {}

    def add_entry(self, module_name, test_name, query_count):
        module_data = self._data.setdefault(module_name, {})
        module_data[test_name] = {'query-count': query_count}

    def save_json(self):
        with open(self.save_path, 'w') as fp:
            json.dump(self._data, fp, indent=2)

    def finish(self):
        """Serialize and export the test data if performance tests were run."""

        if not self._data:
            return

        self.save_json()


def pytest_addoption(parser):
    group = parser.getgroup('django-queries')
    group.addoption(
        '--django-db-bench',
        dest='queries_results_save_path',
        action='store',
        default=DEFAULT_RESULT_FILENAME,
        metavar='PATH',
        help='Output file for storing the results. Default: .pytest-queries')


@pytest.mark.tryfirst
def pytest_configure(config):
    """Append the plugin markers to the pytest configuration."""
    config_line = (
       '%s: Mark the test as to have their queries counted.'
       '' % PYTEST_QUERY_COUNT_MARKER)
    config.addinivalue_line('markers', config_line)


@pytest.mark.tryfirst
def pytest_load_initial_conftests(early_config, parser, args):
    _set_session(
        early_config,
        _Session(early_config.known_args_namespace.queries_results_save_path))


@pytest.hookimpl(hookwrapper=True)
def pytest_sessionfinish(session, exitstatus):
    _get_session(session).finish()
    yield


@pytest.fixture(autouse=True)
def _pytest_query_marker(request):
    """Use the fixture to count the queries on the current node if it's
    marked with 'count_queries'."""
    marker = request.node.get_closest_marker(PYTEST_QUERY_COUNT_MARKER)
    if marker:
        request.getfixturevalue('count_queries')


@pytest.fixture
def count_queries(request):
    """Wrap a test to count the number of performed queries."""
    from django.db import connection

    with CaptureQueriesContext(connection) as context:
        yield context
        query_count = len(context)

    module = request.node.module.__name__
    bench_name = request.node.name
    _get_session(request).add_entry(module, bench_name, query_count)
